@@ -1,15 +1,19 @@
 # -*- coding: utf-8 -*-
 
 from __future__ import print_function
+from __future__ import absolute_import
 import os
 import sys
-import queue
+import six.moves.queue
 import shutil
 import platform
 import tempfile
 import warnings
 import threading
 import subprocess
+import six
+from six.moves import range
+from six.moves import zip
 
 try:
     import multiprocessing as mp
@@ -34,7 +38,7 @@ from tables.flavor import all_flavors, array_of_flavor
 from tables.parameters import NODE_CACHE_SLOTS
 from tables.description import descr_from_dtype, dtype_from_descr
 from tables.tests import common
-from tables.tests.common import unittest
+from tables.tests.common import unittest, test_filename
 from tables.tests.common import PyTablesTestCase as TestCase
 
 
@@ -153,7 +157,7 @@ class OpenFileTestCase(common.TempFileMixin, TestCase):
     def test00_newFile_unicode_filename(self):
         temp_dir = tempfile.mkdtemp()
         try:
-            h5fname = str(os.path.join(temp_dir, 'test.h5'))
+            h5fname = six.text_type(os.path.join(temp_dir, 'test.h5'))
             with tables.open_file(h5fname, 'w') as h5file:
                 self.assertTrue(h5file, tables.File)
         finally:
@@ -1276,7 +1280,7 @@ class CheckFileTestCase(common.TempFileMixin, TestCase):
         """Checking opening of a generic HDF5 file."""
 
         # Open an existing generic HDF5 file
-        h5fname = self._testFilename("ex-noattr.h5")
+        h5fname = test_filename("ex-noattr.h5")
         with tables.open_file(h5fname, mode="r") as h5file:
             # Check for some objects inside
 
@@ -1310,7 +1314,7 @@ class CheckFileTestCase(common.TempFileMixin, TestCase):
         # uncommented in Group.py!                                        #
         ###################################################################
 
-        h5fname = self._testFilename('smpl_unsupptype.h5')
+        h5fname = test_filename('smpl_unsupptype.h5')
         with tables.open_file(h5fname) as h5file:
             with self.assertWarns(UserWarning):
                 node = h5file.get_node('/CompoundChunked')
@@ -1320,7 +1324,7 @@ class CheckFileTestCase(common.TempFileMixin, TestCase):
         """Checking opening of HDF5 files containing scalar dataset of
         UnImlemented type."""
 
-        with tables.open_file(self._testFilename("scalar.h5")) as h5file:
+        with tables.open_file(test_filename("scalar.h5")) as h5file:
             with self.assertWarns(UserWarning):
                 node = h5file.get_node('/variable length string')
             self.assertTrue(isinstance(node, UnImplemented))
@@ -1329,7 +1333,7 @@ class CheckFileTestCase(common.TempFileMixin, TestCase):
         """Checking that an UnImplemented object cannot be copied."""
 
         # Open an existing generic HDF5 file
-        h5fname = self._testFilename("smpl_unsupptype.h5")
+        h5fname = test_filename("smpl_unsupptype.h5")
         with tables.open_file(h5fname, mode="r") as h5file:
             self.assertWarns(UserWarning, h5file.get_node, '/CompoundChunked')
             with warnings.catch_warnings():
@@ -1350,7 +1354,7 @@ class CheckFileTestCase(common.TempFileMixin, TestCase):
         # Open an existing generic HDF5 file
         # We don't need to wrap this in a try clause because
         # it has already been tried and the warning will not happen again
-        h5fname2 = self._testFilename("ex-noattr.h5")
+        h5fname2 = test_filename("ex-noattr.h5")
         with tables.open_file(h5fname2, mode="r") as h5file2:
                 # An unsupported object (the deprecated H5T_ARRAY type in
             # Array, from pytables 0.8 on)
@@ -1402,7 +1406,7 @@ class ThreadingTestCase(common.TempFileMixin, TestCase):
                 q.put('OK')
 
         threads = []
-        q = queue.Queue()
+        q = six.moves.queue.Queue()
         for i in range(10):
             t = threading.Thread(target=run, args=(filename, q))
             t.start()
@@ -1631,7 +1635,7 @@ class StateTestCase(common.TempFileMixin, TestCase):
         self.h5file.create_group('/', 'test1')
         self.h5file.create_group('/', 'test2')
         self.h5file.close()
-        self.assertRaises(ClosedFileError, self.h5file.walk_nodes().__next__)
+        self.assertRaises(ClosedFileError, next, self.h5file.walk_nodes())
 
     def test15_fileAttrClosed(self):
         """Test setting and deleting a node attribute in a closed file."""
@@ -1923,7 +1927,7 @@ class FlavorTestCase(common.TempFileMixin, TestCase):
 @unittest.skipIf(sys.getfilesystemencoding() != 'utf-8',
                  'need utf-8 file-system encoding')
 class UnicodeFilename(common.TempFileMixin, TestCase):
-    unicode_prefix = 'para\u0140lel'
+    unicode_prefix = u'para\u0140lel'
 
     def _getTempFileName(self):
         return tempfile.mktemp(prefix=self.unicode_prefix, suffix='.h5')
@@ -1975,7 +1979,7 @@ class UnicodeFilename(common.TempFileMixin, TestCase):
         self.h5file = tables.open_file(self.h5fname, "a")
         root = self.h5file.root
         group = self.h5file.create_group(root, 'face_data')
-        array_name = 'data at 40\N{DEGREE SIGN}C'
+        array_name = u'data at 40\N{DEGREE SIGN}C'
         data = numpy.sinh(numpy.linspace(-1.4, 1.4, 500))
         with warnings.catch_warnings():
             warnings.simplefilter('ignore', NaturalNameWarning)
@@ -1983,6 +1987,51 @@ class UnicodeFilename(common.TempFileMixin, TestCase):
         group = self.h5file.create_group(root, 'vertex_data')
 
 
+@unittest.skipIf(sys.version_info < (3, 6),
+                 'PEP 519 was implemented in Python 3.6')
+class PathLikeFilename(common.TempFileMixin, TestCase):
+
+    def _getTempFileName(self):
+        from pathlib import Path
+        return Path(tempfile.mktemp(suffix='.h5'))
+
+    def setUp(self):
+        super(PathLikeFilename, self).setUp()
+
+        self.test = self.h5file.create_array('/', 'test', [1, 2])
+
+        # So as to check the reading
+        self._reopen()
+
+    def test01(self):
+        """Checking creating a file with a PathLike object as the filename."""
+
+        test = self.h5file.root.test
+        if common.verbose:
+            print("Filename:", self.h5fname)
+            print("Array:", test[:])
+            print("Should look like:", [1, 2])
+        self.assertEqual(test[:], [1, 2], "Values does not match.")
+
+    def test02(self):
+        """Checking  tables.is_hdf5_file with a PathLike object as the filename."""
+
+        self.h5file.close()
+        if common.verbose:
+            print("Filename:", self.h5fname)
+            print(" tables.is_hdf5_file?:", tables.is_hdf5_file(self.h5fname))
+        self.assertTrue(tables.is_hdf5_file(self.h5fname))
+
+    def test03(self):
+        """Checking is_pytables_file with a PathLike object as the filename."""
+
+        self.h5file.close()
+        if common.verbose:
+            print("Filename:", self.h5fname)
+            print("is_pytables_file?:", tables.is_pytables_file(self.h5fname))
+        self.assertNotEqual(tables.is_pytables_file(self.h5fname), False)
+
+
 class FilePropertyTestCase(TestCase):
     def setUp(self):
         super(FilePropertyTestCase, self).setUp()
@@ -2060,7 +2109,7 @@ class FilePropertyTestCase(TestCase):
 # Test for reading a file that uses Blosc and created on a big-endian platform
 @unittest.skipIf(not common.blosc_avail, 'Blosc not available')
 class BloscBigEndian(common.TestFileMixin, TestCase):
-    h5fname = TestCase._testFilename("blosc_bigendian.h5")
+    h5fname = test_filename("blosc_bigendian.h5")
 
     def test00_bigendian(self):
         """Checking compatibility with Blosc on big-endian machines."""
@@ -2385,13 +2434,13 @@ class TestDescription(TestCase):
         # see gh-42
         # the name used is a valid ASCII identifier passed as unicode
         # string
-        d = {str('name'): tables.Int16Col()}
+        d = {six.text_type('name'): tables.Int16Col()}
         descr = Description(d)
         self.assertEqual(sorted(descr._v_names), sorted(d.keys()))
         self.assertTrue(isinstance(descr._v_dtype, numpy.dtype))
         keys = []
         for key in list(d.keys()):
-            if isinstance(key, str):
+            if isinstance(key, six.text_type):
                 keys.append(key.encode())
             else:
                 keys.append(key)
@@ -2462,12 +2511,12 @@ class TestCol(TestCase):
 
 class TestSysattrCompatibility(TestCase):
     def test_open_python2(self):
-        h5fname = self._testFilename("python2.h5")
+        h5fname = test_filename("python2.h5")
         with tables.open_file(h5fname, "r") as h5file:
             self.assertTrue(h5file.isopen)
 
     def test_open_python3(self):
-        h5fname = self._testFilename("python3.h5")
+        h5fname = test_filename("python3.h5")
         with tables.open_file(h5fname, "r") as h5file:
             self.assertTrue(h5file.isopen)
 
@@ -2487,6 +2536,7 @@ def suite():
         theSuite.addTest(unittest.makeSuite(StateTestCase))
         theSuite.addTest(unittest.makeSuite(FlavorTestCase))
         theSuite.addTest(unittest.makeSuite(UnicodeFilename))
+        theSuite.addTest(unittest.makeSuite(PathLikeFilename))
         theSuite.addTest(unittest.makeSuite(FilePropertyTestCase))
         theSuite.addTest(unittest.makeSuite(BloscBigEndian))
         theSuite.addTest(unittest.makeSuite(BloscSubprocess))
